@@ -1,21 +1,27 @@
 package HTML::Lint;
 
+use warnings;
+use strict;
+
+use HTML::Lint::Error;
+use HTML::Entities ();
+
 =head1 NAME
 
 HTML::Lint - check for HTML errors in a string or file
 
 =head1 VERSION
 
-Version 2.00
+Version 2.11_01
 
 =cut
 
-our $VERSION = '2.00';
+our $VERSION = '2.11_01';
 
 =head1 SYNOPSIS
 
     my $lint = HTML::Lint->new;
-    $lint->only_types( HTML::Lint::STRUCTURE );
+    $lint->only_types( HTML::Lint::Error::STRUCTURE );
 
     $lint->parse( $data );
     $lint->parse_file( $filename );
@@ -30,8 +36,8 @@ HTML::Lint also comes with a wrapper program called F<weblint> that handles
 linting from the command line:
 
     $ weblint http://www.cnn.com/
-    http://www.cnn.com/ (395:83) <IMG> tag has no HEIGHT and WIDTH attributes.
-    http://www.cnn.com/ (395:83) <IMG> does not have ALT text defined
+    http://www.cnn.com/ (395:83) <IMG SRC="spacer.gif"> tag has no HEIGHT and WIDTH attributes.
+    http://www.cnn.com/ (395:83) <IMG SRC="goofus.gif"> does not have ALT text defined
     http://www.cnn.com/ (396:217) Unknown element <nobr>
     http://www.cnn.com/ (396:241) </nobr> with no opening <nobr>
     http://www.cnn.com/ (842:7) target attribute in <a> is repeated
@@ -46,9 +52,6 @@ Apache F<error_log>.
 
 =cut
 
-use strict;
-use HTML::Lint::Error;
-
 =head1 METHODS
 
 NOTE: Some of these methods mirror L<HTML::Parser>'s methods, but HTML::Lint
@@ -64,7 +67,7 @@ C<only_types> parm.
 
 If you want more than one, you must pass an arrayref:
 
-    my $lint = HTML::Lint->new( 
+    my $lint = HTML::Lint->new(
         only_types => [HTML::Lint::Error::STRUCTURE, HTML::Lint::Error::FLUFF] );
 
 =cut
@@ -75,13 +78,12 @@ sub new {
 
     my $self = {
         _errors => [],
-        _types => [],
+        _types  => [],
     };
-    $self->{_parser} = HTML::Lint::Parser->new( sub { $self->gripe( @_ ) } );
     bless $self, $class;
 
     if ( my $only = $args{only_types} ) {
-        $self->only_types( ref $only eq "ARRAY" ? @$only : $only );
+        $self->only_types( ref $only eq 'ARRAY' ? @{$only} : $only );
         delete $args{only_types};
     }
 
@@ -90,7 +92,24 @@ sub new {
     return $self;
 }
 
-=head2 $lint->parse( $chunk )
+=head2 $lint->parser()
+
+Returns the parser object for this object, creating one if necessary.
+
+=cut
+
+sub parser {
+    my $self = shift;
+
+    if ( not $self->{_parser} ) {
+        $self->{_parser} = HTML::Lint::Parser->new( sub { $self->gripe( @_ ) } );
+        $self->{_parser}->ignore_elements( qw(script style) );
+    }
+
+    return $self->{_parser};
+}
+
+=head2 $lint->parse( $text )
 
 =head2 $lint->parse( $code_ref )
 
@@ -102,7 +121,8 @@ See L<HTML::Parser>'s C<parse_file> method for details.
 
 sub parse {
     my $self = shift;
-    return $self->_parser->parse( @_ );
+
+    return $self->parser->parse( @_ );
 }
 
 =head2 $lint->parse_file( $file )
@@ -115,7 +135,7 @@ See L<HTML::Parser>'s C<parse_file> method for details.
 
 sub parse_file {
     my $self = shift;
-    return $self->_parser->parse_file( @_ );
+    return $self->parser->parse_file( @_ );
 }
 
 =head2 $lint->eof
@@ -132,9 +152,9 @@ sub eof {
     my $self = shift;
 
     my $rc;
-    my $parser = $self->_parser;
+    my $parser = $self->parser;
     if ( $parser ) {
-        $rc = $self->_parser->eof(@_);
+        $rc = $self->parser->eof(@_);
         delete $self->{_parser};
     }
 
@@ -171,6 +191,8 @@ sub clear_errors {
     my $self = shift;
 
     $self->{_errors} = [];
+
+    return;
 }
 
 =head2 $lint->only_types( $type1[, $type2...] )
@@ -191,6 +213,8 @@ sub only_types {
     my $self = shift;
 
     $self->{_types} = [@_];
+
+    return;
 }
 
 =head2 $lint->gripe( $errcode, [$key1=>$val1, ...] )
@@ -212,22 +236,22 @@ in case, here you go.
 sub gripe {
     my $self = shift;
 
-    my $parser = $self->_parser;
-
-    my $error = new HTML::Lint::Error(
-        $self->{_file}, $parser->{_line}, $parser->{_column}, @_ );
+    my $error = HTML::Lint::Error->new(
+        $self->{_file}, $self->parser->{_line}, $self->parser->{_column}, @_ );
 
     my @keeps = @{$self->{_types}};
     if ( !@keeps || $error->is_type(@keeps) ) {
         push( @{$self->{_errors}}, $error );
     }
+
+    return;
 }
 
 =head2 $lint->newfile( $filename )
 
-Call C<newfile()> whenever you switch to another file in a batch of 
-linting.  Otherwise, the object thinks everything is from the same file.
-Note that the list of errors is NOT cleared.
+Call C<newfile()> whenever you switch to another file in a batch
+of linting.  Otherwise, the object thinks everything is from the
+same file.  Note that the list of errors is NOT cleared.
 
 Note that I<$filename> does NOT need to match what's put into parse()
 or parse_file().  It can be a description, a URL, or whatever.
@@ -239,7 +263,6 @@ sub newfile {
     my $file = shift;
 
     delete $self->{_parser};
-    $self->{_parser} = HTML::Lint::Parser->new( sub { $self->gripe( @_ ) } );
     $self->{_file} = $file;
     $self->{_line} = 0;
     $self->{_column} = 0;
@@ -248,12 +271,6 @@ sub newfile {
     return $self->{_file};
 } # newfile
 
-sub _parser {
-    my $self = shift;
-
-    return $self->{_parser};
-}
-
 =pod
 
 HTML::Lint::Parser is a class only for this module.
@@ -268,7 +285,7 @@ use HTML::Tagset 3.03;
 use HTML::Lint::HTML4 qw( %isKnownAttribute %isRequired %isNonrepeatable %isObsolete );
 use HTML::Entities qw( %char2entity );
 
-our @ISA = qw( HTML::Parser );
+use base 'HTML::Parser';
 
 sub new {
     my $class = shift;
@@ -277,12 +294,13 @@ sub new {
     my $self =
         HTML::Parser->new(
             api_version => 3,
-            start_document_h    => [ \&_start_document, 'self' ],
-            end_document_h      => [ \&_end_document,   'self,line,column' ],
-            start_h             => [ \&_start,          'self,tagname,line,column,@attr' ],
-            end_h               => [ \&_end,            'self,tagname,line,column,@attr' ],
-            text_h              => [ \&_text,           'self,text' ],
-            strict_names => 1,
+            start_document_h   => [ \&_start_document, 'self' ],
+            end_document_h     => [ \&_end_document,   'self,line,column' ],
+            start_h            => [ \&_start,          'self,tagname,line,column,@attr' ],
+            end_h              => [ \&_end,            'self,tagname,line,column,tokenpos,@attr' ],
+            text_h             => [ \&_text,           'self,text' ],
+            strict_names       => 0,
+            empty_element_tags => 1,
         );
     bless $self, $class;
 
@@ -299,7 +317,6 @@ sub gripe {
 }
 
 sub _start_document {
-    my $self = shift;
 }
 
 sub _end_document {
@@ -310,6 +327,8 @@ sub _end_document {
             $self->gripe( 'doc-tag-required', tag => $tag );
         }
     }
+
+    return;
 }
 
 sub _start {
@@ -342,7 +361,7 @@ sub _start {
         if ( $isNonrepeatable{$tag} ) {
             $self->gripe( 'elem-nonrepeatable',
                             tag => $tag,
-                            where => HTML::Lint::Error::where(@$where)
+                            where => HTML::Lint::Error::where( @{$where} )
                         );
         }
     }
@@ -355,36 +374,92 @@ sub _start {
     if ( $self->can($tagfunc) ) {
         $self->$tagfunc( $tag, @attr );
     }
+
+    return;
 }
 
 sub _text {
     my ($self,$text) = @_;
 
+    while ( $text =~ /&(?![#0-9a-z])/ig ) {
+        $self->gripe( 'text-use-entity', char => '&', entity => '&amp;' );
+    }
+
     while ( $text =~ /([^\x09\x0A\x0D -~])/g ) {
         my $bad = $1;
         $self->gripe(
-            'text-use-entity', 
+            'text-use-entity',
                 char => sprintf( '\x%02lX', ord($bad) ),
                 entity => $char2entity{ $bad },
         );
     }
+
+    if ( not $self->{_unclosed_entities_regex} ) {
+        # Get Gisle's list
+        my @entities = sort keys %HTML::Entities::entity2char;
+
+        # Strip his semicolons
+        s/;$// for @entities;
+
+        # Build a regex
+        my $entities = join( '|', @entities );
+        $self->{_unclosed_entities_regex} = qr/&($entities)(?!;)/;
+
+        $self->{_entity_lookup} = { map { ($_,1) } @entities };
+    }
+
+    while ( $text =~ m/$self->{_unclosed_entities_regex}/g ) {
+        my $ent = $1;
+        $self->gripe( 'text-unclosed-entity', entity => "&$ent;" );
+    }
+
+    while ( $text =~ m/&([^;]+);/g ) {
+        my $ent = $1;
+
+        # Numeric entities are fine, if they're not too large.
+        if ( $ent =~ /^#(\d+)$/ ) {
+            if ( $1 > 65536 ) {
+                $self->gripe( 'text-invalid-entity', entity => "&$ent;" );
+            }
+            next;
+        }
+
+        # Hex entities are fine, if they're not too large.
+        if ( $ent =~ /^#x([\dA-F]+)$/i ) {
+            if ( length($1) > 4 ) {
+                $self->gripe( 'text-invalid-entity', entity => "&$ent;" );
+            }
+            next;
+        }
+
+        # If it's not a numeric entity, then check the lookup table.
+        if ( !exists $self->{_entity_lookup}{$ent} ) {
+            $self->gripe( 'text-unknown-entity', entity => "&$ent;" );
+        }
+    }
+
+    return;
 }
 
 sub _end {
-    my ($self,$tag,$line,$column,@attr) = @_;
+    my ($self,$tag,$line,$column,$tokenpos,@attr) = @_;
 
     $self->{_line} = $line;
     $self->{_column} = $column;
 
-    if ( $HTML::Tagset::emptyElement{ $tag } ) {
+    if ( !$tokenpos ) {
+        # This is a dummy end event for something like <img />.
+        # Do nothing.
+    }
+    elsif ( $HTML::Tagset::emptyElement{ $tag } ) {
         $self->gripe( 'elem-empty-but-closed', tag => $tag );
     }
     else {
         if ( $self->_in_context($tag) ) {
             my @leftovers = $self->_element_pop_back_to($tag);
             for ( @leftovers ) {
-                my ($tag,$line,$col) = @$_;
-                $self->gripe( 'elem-unclosed', tag => $tag, 
+                my ($tag,$line,$col) = @{$_};
+                $self->gripe( 'elem-unclosed', tag => $tag,
                         where => HTML::Lint::Error::where($line,$col) )
                         unless $HTML::Tagset::optionalEndTag{$tag};
             } # for
@@ -399,6 +474,8 @@ sub _end {
     if ( $self->can($tagfunc) ) {
         $self->$tagfunc( $tag, $line );
     }
+
+    return;
 }
 
 sub _element_push {
@@ -406,6 +483,8 @@ sub _element_push {
     for ( @_ ) {
         push( @{$self->{_stack}}, [$_,$self->{_line},$self->{_column}] );
     } # while
+
+    return;
 }
 
 sub _find_tag_in_stack {
@@ -413,7 +492,7 @@ sub _find_tag_in_stack {
     my $tag = shift;
     my $stack = $self->{_stack};
 
-    my $offset = @$stack - 1;
+    my $offset = @{$stack} - 1;
     while ( $offset >= 0 ) {
         if ( $stack->[$offset][0] eq $tag ) {
             return $offset;
@@ -448,27 +527,27 @@ sub _in_context {
 sub _start_img {
     my ($self,$tag,%attr) = @_;
 
-    my ($h,$w) = @attr{qw( height width )};
+    my ($h,$w,$src) = @attr{qw( height width src )};
     if ( defined $h && defined $w ) {
         # Check sizes
     }
     else {
-        $self->gripe( "elem-img-sizes-missing" );
+        $self->gripe( 'elem-img-sizes-missing', src=>$src );
     }
     if ( not defined $attr{alt} ) {
-        $self->gripe( "elem-img-alt-missing" );
+        $self->gripe( 'elem-img-alt-missing', src=>$src );
     }
+
+    return;
 }
 
 =head1 BUGS, WISHES AND CORRESPONDENCE
 
-Please feel free to email me at andy@petdance.com.  I'm glad to help as
-best I can, and I'm always interested in bugs, suggestions and patches.
+All bugs and requests are now being handled through GitHub.
 
-Please report any bugs or feature requests to
-C<< <bug-html-lint at rt.cpan.org> >>, or through the web interface at
-L<http://rt.cpan.org>.  I will be notified, and then you'll automatically
-be notified of progress on your bug as I make changes.
+    https://github.com/petdance/html-lint/issues
+
+DO NOT send bug reports to http://rt.cpan.org/ or http://code.google.com/
 
 =head1 TODO
 
@@ -492,12 +571,14 @@ be notified of progress on your bug as I make changes.
 
 =back
 
-=head1 LICENSE
+=head1 COPYRIGHT & LICENSE
 
-Copyright 2005 Andy Lester, All Rights Reserved.
+Copyright 2005-2012 Andy Lester.
 
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
+This program is free software; you can redistribute it and/or modify
+it under the terms of the Artistic License v2.0.
+
+http://www.opensource.org/licenses/Artistic-2.0
 
 Please note that these modules are not products of or supported by the
 employers of the various contributors to the code.
@@ -508,3 +589,4 @@ Andy Lester, andy at petdance.com
 
 =cut
 
+1;
