@@ -1,8 +1,10 @@
 package HTML::Lint::Error;
 
+use warnings;
 use strict;
 
 use base 'Exporter';
+
 our @EXPORT = ();
 our @EXPORT_OK = qw( STRUCTURE HELPER FLUFF );
 our %EXPORT_TAGS = ( types => [@EXPORT_OK] );
@@ -98,7 +100,7 @@ sub _expand_error {
     my $specs = $errors{$errcode};
     my $str;
     if ( $specs ) {
-        ($str, $self->{_type}) = @$specs;
+        ($str, $self->{_type}) = @{$specs};
     }
     else {
         $str = "Unknown code: $errcode";
@@ -170,7 +172,7 @@ sub where {
         $col = $self->column;
     }
     $col ||= 0;
-    return sprintf( "(%s:%s)", $line, $col + 1 );
+    return sprintf( '(%s:%s)', $line, $col + 1 );
 }
 
 =head2 as_string()
@@ -182,7 +184,7 @@ Returns a nicely-formatted string for printing out to stdout or some similar use
 sub as_string {
     my $self = shift;
 
-    return sprintf( "%s %s %s", $self->file, $self->where, $self->errtext );
+    return sprintf( '%s %s %s', $self->file, $self->where, $self->errtext );
 }
 
 =head2 file()
@@ -241,21 +243,24 @@ Andy Lester, C<andy at petdance.com>
 
 # Generic element stuff
 %errors = (
-    'elem-unknown' =>           ['Unknown element <${tag}>', STRUCTURE],
-    'elem-unopened' =>          ['</${tag}> with no opening <${tag}>', STRUCTURE],
-    'elem-unclosed' =>          ['<${tag}> at ${where} is never closed', STRUCTURE],
-    'elem-empty-but-closed' =>  ['<${tag}> is not a container -- </${tag}> is not allowed', STRUCTURE],
+    'elem-unknown'           => ['Unknown element <${tag}>', STRUCTURE],
+    'elem-unopened'          => ['</${tag}> with no opening <${tag}>', STRUCTURE],
+    'elem-unclosed'          => ['<${tag}> at ${where} is never closed', STRUCTURE],
+    'elem-empty-but-closed'  => ['<${tag}> is not a container -- </${tag}> is not allowed', STRUCTURE],
 
-    'elem-img-sizes-missing' => ['<IMG> tag has no HEIGHT and WIDTH attributes.', HELPER],
-    'elem-img-alt-missing' =>   ['<IMG> does not have ALT text defined', HELPER],
-    'elem-nonrepeatable' =>     ['<${tag}> is not repeatable, but already appeared at ${where}', STRUCTURE],
+    'elem-img-sizes-missing' => ['<IMG SRC="${src}"> tag has no HEIGHT and WIDTH attributes.', HELPER],
+    'elem-img-alt-missing'   => ['<IMG SRC="${src}"> does not have ALT text defined', HELPER],
+    'elem-nonrepeatable'     => ['<${tag}> is not repeatable, but already appeared at ${where}', STRUCTURE],
 
-    'doc-tag-required' =>       ['<${tag}> tag is required', STRUCTURE],
+    'doc-tag-required'       => ['<${tag}> tag is required', STRUCTURE],
 
-    'attr-repeated' =>          ['${attr} attribute in <${tag}> is repeated', STRUCTURE],
-    'attr-unknown' =>           ['Unknown attribute "${attr}" for tag <${tag}>', FLUFF],
+    'attr-repeated'          => ['${attr} attribute in <${tag}> is repeated', STRUCTURE],
+    'attr-unknown'           => ['Unknown attribute "${attr}" for tag <${tag}>', FLUFF],
 
-    'text-use-entity' =>        ['Invalid character ${char} should be written as ${entity}', STRUCTURE],
+    'text-invalid-entity'    => ['Entity ${entity} is invalid', STRUCTURE],
+    'text-unclosed-entity'   => ['Entity ${entity} is missing its closing semicolon', STRUCTURE],
+    'text-unknown-entity'    => ['Entity ${entity} is unknown', STRUCTURE],
+    'text-use-entity'        => ['Character "${char}" should be written as ${entity}', STRUCTURE],
 );
 
 1; # happy
