@@ -1,11 +1,12 @@
 package Test::HTML::Lint;
 
+use warnings;
 use strict;
 
 use Test::Builder;
 use Exporter;
 
-use HTML::Lint 2.00;
+use HTML::Lint;
 
 use vars qw( @ISA $VERSION @EXPORT );
 
@@ -17,11 +18,11 @@ Test::HTML::Lint - Test::More-style wrapper around HTML::Lint
 
 =head1 VERSION
 
-Version 2.00
+Version 2.11_01
 
 =cut
 
-$VERSION = '2.00';
+$VERSION = '2.11_01';
 
 my $Tester = Test::Builder->new;
 
@@ -57,6 +58,8 @@ sub import {
     $Tester->plan(@_);
 
     $self->export_to_level(1, $self, @EXPORT);
+
+    return;
 }
 
 =head2 html_ok( [$lint, ] $html, $name )
@@ -84,30 +87,32 @@ will clear its errors before using it.
 sub html_ok {
     my $lint;
 
-    if ( ref($_[0]) eq "HTML::Lint" ) {
-	$lint = shift;
-	$lint->newfile();
-	$lint->clear_errors();
-    } else {
-	$lint = HTML::Lint->new;
+    if ( ref($_[0]) eq 'HTML::Lint' ) {
+        $lint = shift;
+        $lint->newfile();
+        $lint->clear_errors();
+    }
+    else {
+        $lint = HTML::Lint->new;
     }
     my $html = shift;
     my $name = shift;
 
     my $ok = defined $html;
     if ( !$ok ) {
-	$Tester->ok( 0, $name );
-    } else {
-	$lint->parse( $html );
-	my $nerr = scalar $lint->errors;
-	$ok = !$nerr;
+        $Tester->ok( 0, $name );
+    }
+    else {
+        $lint->parse( $html );
+        my $nerr = scalar $lint->errors;
+        $ok = !$nerr;
         $Tester->ok( $ok, $name );
-	if ( !$ok ) {
-	    my $msg = "Errors:";
-	    $msg .= " $name" if $name;
-	    $Tester->diag( $msg );
-	    $Tester->diag( $_->as_string ) for $lint->errors;
-	}
+        if ( !$ok ) {
+            my $msg = 'Errors:';
+            $msg .= " $name" if $name;
+            $Tester->diag( $msg );
+            $Tester->diag( $_->as_string ) for $lint->errors;
+        }
     }
 
     return $ok;
@@ -115,10 +120,12 @@ sub html_ok {
 
 =head1 BUGS
 
-Please report any bugs or feature requests to C<bug-html-lint@rt.cpan.org>,
-or through the web interface at L<http://rt.cpan.org>.  I will be
-notified, and then you'll automatically be notified of progress on
-your bug as I make changes.
+All bugs and requests are now being handled through GitHub.
+
+    https://github.com/petdance/html-lint/issues
+
+DO NOT send bug reports to http://rt.cpan.org/ or http://code.google.com/
+
 
 =head1 TO DO
 
@@ -129,7 +136,7 @@ C<html_page_ok()>.
 If you think this module should do something that it doesn't do at the
 moment please let me know.
 
-=head1 ACKNOWLEGEMENTS
+=head1 ACKNOWLEDGEMENTS
 
 Thanks to chromatic and Michael G Schwern for the excellent Test::Builder,
 without which this module wouldn't be possible.
@@ -137,16 +144,19 @@ without which this module wouldn't be possible.
 Thanks to Adrian Howard for writing Test::Exception, from which most of
 this module is taken.
 
-=head1 LICENSE
+=head1 COPYRIGHT & LICENSE
 
-Copyright 2003 Andy Lester, All Rights Reserved.
+Copyright 2005-2011 Andy Lester.
 
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
+This program is free software; you can redistribute it and/or modify
+it under the terms of the Artistic License v2.0.
+
+http://www.opensource.org/licenses/Artistic-2.0
 
 Please note that these modules are not products of or supported by the
 employers of the various contributors to the code.
 
+
 =head1 AUTHOR
 
 Andy Lester, C<andy@petdance.com>
